#import "GBShadowsView.h"

@interface GBShadowsView ()
@property (nonatomic,strong) CAShapeLayer *roundedRectWithDropShadowLayer;
@property (nonatomic,strong) CAShapeLayer *roundedRectWithInnerShadowLayer;
@property (nonatomic,strong) CAShapeLayer *innerShadowMaskLayer;
@property (nonatomic,strong) CAShapeLayer *innerShadowLayer;
@end

@implementation GBShadowsView

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self createScene];
    }
    return self;
}

- (void)createScene {
    /*
     * Drop shadow
     */

    // Create a CAShapeLayer with a yellow shape, and add it to the view's layer
    self.roundedRectWithDropShadowLayer = [CAShapeLayer new];
    self.roundedRectWithDropShadowLayer.fillColor = [UIColor yellowColor].CGColor;
    [self.layer addSublayer:self.roundedRectWithDropShadowLayer];
    
    // Give the shape a shadow
    self.roundedRectWithDropShadowLayer.shadowColor = [UIColor blackColor].CGColor;
    self.roundedRectWithDropShadowLayer.shadowOpacity = 0.5;
    self.roundedRectWithDropShadowLayer.shadowRadius = 15;

    /*
     * Inner shadow
     */
    
    // Create a CAShapeLayer with a yellow shape, and add it to the view's layer
    self.roundedRectWithInnerShadowLayer = [CAShapeLayer new];
    self.roundedRectWithInnerShadowLayer.fillColor = [UIColor yellowColor].CGColor;
    [self.layer addSublayer:self.roundedRectWithInnerShadowLayer];

    // Create a mask layer with the same shape
    self.innerShadowMaskLayer = [CAShapeLayer new];

    // Add an inner shadow layer
    self.innerShadowLayer = [CAShapeLayer new];
    self.innerShadowLayer.shadowColor = [UIColor blackColor].CGColor;
    self.innerShadowLayer.shadowOpacity = 0.7;
    self.innerShadowLayer.shadowRadius = 10;
    self.innerShadowLayer.mask = self.innerShadowMaskLayer;
    [self.layer addSublayer:self.innerShadowLayer];
}

- (void)layoutSublayersOfLayer:(CALayer *)layer {
    if (layer != self.layer) return;

    // Create two frames so we can divide the screen in two
    CGFloat halfHeight = floorf(self.bounds.size.height / 2.0);
    CGRect topHalf = self.bounds;
    topHalf.size.height = halfHeight;
    CGRect bottomHalf = self.bounds;
    bottomHalf.size.height = halfHeight;
    bottomHalf.origin.y = halfHeight;
    CGRect bottomHalfBounds = bottomHalf;
    bottomHalf.origin.y = 0;

    /*
     * Drop shadow
     */

    // Layout the rounded rect layer and give it a rounded rectangle shape
    self.roundedRectWithDropShadowLayer.frame = topHalf;
    self.roundedRectWithDropShadowLayer.path = [GBShapes
                                  roundedRectShapeInFrame:CGRectInset(topHalf, 40, 40)].CGPath;

    // Give the shadow a shape
    self.roundedRectWithDropShadowLayer.shadowPath = self.roundedRectWithDropShadowLayer.path;

    /*
     * Inner shadow
     */

    // Layout rounded rect and give it a rounded rectangle shape
    self.roundedRectWithInnerShadowLayer.frame = bottomHalf;
    self.roundedRectWithInnerShadowLayer.path = [GBShapes
                                  roundedRectShapeInFrame:CGRectInset(bottomHalfBounds, 40, 40)].CGPath;

    // Layout the shadow mask and give it the same shape as the rounded rect
    self.innerShadowMaskLayer.frame = bottomHalf;
    self.innerShadowMaskLayer.path = self.roundedRectWithInnerShadowLayer.path;

    // Layout inner shadow
    self.innerShadowLayer.frame = bottomHalf;

    // Create a path that traces the border of the shape, 20 pixels thick
    CGPathRef innerShadowPath = CGPathCreateCopyByStrokingPath(self.roundedRectWithInnerShadowLayer.path,
                                                               NULL,
                                                               20,
                                                               kCGLineCapRound,
                                                               kCGLineJoinRound,
                                                               0);

    // Use that path as a shadow path for the inner shadow layer
    self.innerShadowLayer.shadowPath = innerShadowPath;
    CGPathRelease(innerShadowPath);
}

#pragma mark Light source handling

- (void)setLightSourceWithTouch:(UITouch *)touch {
    // Compute offsets for the shadows. This isn't very realistic as we're
    // not considering perspective
    CGPoint location = [touch locationInView:self];
    CGFloat xFraction = location.x / self.bounds.size.width;
    CGFloat yFraction = location.y / self.bounds.size.height;
    CGFloat xOffset = -30 + (1 - xFraction) * 60;
    CGFloat yOffset = -30 + (1 - yFraction) * 60;

    // Set the new shadow offsets
    self.roundedRectWithDropShadowLayer.shadowOffset = CGSizeMake(xOffset, yOffset);
    self.innerShadowLayer.shadowOffset = CGSizeMake(xOffset * 0.08, yOffset * 0.08);
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    [self setLightSourceWithTouch:[[touches allObjects] objectAtIndex:0]];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    [self setLightSourceWithTouch:[[touches allObjects] objectAtIndex:0]];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    [self setLightSourceWithTouch:[[touches allObjects] objectAtIndex:0]];
}

@end
